
package Transformation.Hough;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.MemoryImageSource;
import java.awt.image.PixelGrabber;
import java.util.Stack;
import Transformation.importClass.DateiFunktionen;

/**
 * Klasse BerechnungHough berechnet aus dem Hough Raum ( Akkumulator )
 * die Linien, die unterhalb des (Maximalsten Peak Ausschlag) * Schwellwert
 * liegen. Den Hough Raum bekommt die Klasse mittels Image Objekt ber den
 * Konstruktor mitgegeben.
 */
public class BerechnungHough implements IBildBearbeitung 
{

    private  double SCHWELLWERT=0.88;	
	
	private Image Bild=null;
	private Image AkkumulatorBild=null;
	private Image newImg = null;
	private int Bildbreite;
	private int Bildhoehe;
	private int Akkubreite;
	private int Akkuhoehe;

	/**
	 * Konstruktor Berechnung HoughRaum. Bekommt den Akkumulator
	 * in Form enes Image Objekts bergeben.
	 * @param bild; das Original Bild
	 * @param Bildbreite; Bildbreite des Original Bildes
	 * @param Bildhoehe; Bildh&ouml;he des Original Bildes
	 * @param Akkumulator; Akkumulator
	 * @param Akkubreite; Bildbreite des Akkumulators
	 * @param Akkuhoehe; Bildhoehe des Akkumulators
	 * @param Schwellwert; Vordefinierter Schwellwert
	 */
	public BerechnungHough(Image bild,
			               int Bildbreite,
			               int Bildhoehe,
			               Image Akkumulator,
			               int Akkubreite,
			               int Akkuhoehe,
			               double Schwellwert)
	{
		this.Bild = bild;
		this.AkkumulatorBild = Akkumulator;
		this.Bildbreite=Bildbreite;
		this.Bildhoehe=Bildhoehe;
		this.Akkubreite=Akkubreite;
		this.Akkuhoehe=Akkuhoehe;
		this.SCHWELLWERT=Schwellwert;

		bildBerechnen();		
	}
	
	/**
	 * Methode berechnet aus dem Akkumulator die Linien.
	 *
	 */
	private void bildBerechnen() 
	{	
	    
		int[] Akkumulator = new int[this.Akkubreite*this.Akkuhoehe];
		int[] Originalbild = new int[this.Bildbreite*this.Bildhoehe];
		
		/*Bereite die Bilder vor. Lass ber den Akkumulator den
		 * PixelGrabber laufen.
		 */
		
		PixelGrabber pg = new PixelGrabber(AkkumulatorBild, 0, 0, this.Akkubreite, this.Akkuhoehe, Akkumulator, 0, this.Akkubreite);
	    try
	    {
	            pg.grabPixels();
	    }
	    catch (InterruptedException e)
	    {
	       System.err.println("interrupted waiting for pixels!");
	       
	    }

		
		/*Das erste was wir machen mssen ist den maximalsten Wert
		herauszufinden. Der Schwellwertfaktor wird mit diesem
		multipliziert->Daraus wird unser Schwellwert fr die
		Peaks.*/
	    
	    //POWERTEST
    	setPowertest();
		int maxPeak=0;
        for(int i=0;i<this.Akkubreite*Akkuhoehe;i++)
        {
        	 //POWERTEST
        	setPowertest();
        	if((Akkumulator[i] & 0x00FFFFFF)>maxPeak)
        	{
        		 //POWERTEST
            	setPowertest();
        		maxPeak=(Akkumulator[i] & 0x00FFFFFF);
        	}
        		
        }
		
		/*
		 * Die Linien werden in einem Stack gespeichert.
		 * Um die Linien mit ihren Start- und Enpunkten
		 * speichern zu knnen, wurde eine eigene Klasse
		 * geschrieben:LineInformation
		 */
		Stack Linien = new Stack();
		
		//Filtere Alle Peaks heraus die ber dem Schwellwert liegen
		for(int winkel=0;winkel<180;winkel++)
		{
			for(int r=0;r<Akkubreite;r++)
			{
				 //POWERTEST
		    	setPowertest();
				if((Akkumulator[(winkel)*Akkubreite+r] & 0x00FFFFFF)>=(maxPeak*SCHWELLWERT))
				{
					//Zurckrechnen.
					// 1. Bestimmen des Ortsvektors
					 //POWERTEST
			    	setPowertest();
					 int MAX_RAD=(new Double(Math.sqrt(this.Bildbreite*this.Bildbreite+this.Bildhoehe*this.Bildhoehe))).intValue();
					 //POWERTEST
				    	setPowertest();
					 int neu_r=r-MAX_RAD;
					 //POWERTEST
				    	setPowertest();
					 double OVx =Math.cos(winkel*Math.PI/180)*neu_r;
					 //POWERTEST
				    	setPowertest();
					 double OVy =Math.sin(winkel*Math.PI/180)*neu_r;
					 //2. Drehung des Winkels um 90 weil wir oben mit der Normalen gearbeitet haben
					 //POWERTEST
				    	setPowertest();
					 int winkelneu=winkel+90;
					 
		
					 
					 //3. Berechnung der Punkte, die unserer Linie am Rand des Bildes macht.
					 Linien.push(new LinieInformationen(
					    (new Double(OVx+Math.cos(winkelneu*Math.PI/180)*-(MAX_RAD*2))).intValue(),
					    (new Double(OVy+Math.sin(winkelneu*Math.PI/180)*-(MAX_RAD*2))).intValue(),
					    (new Double(OVx+Math.cos(winkelneu*Math.PI/180)*(MAX_RAD*2))).intValue(),
					    (new Double(OVy+Math.sin(winkelneu*Math.PI/180)*(MAX_RAD*2))).intValue()));
				}
			}
		}
		
		
		
		//Bearbeite das Original mit den Linieninformationen
		//Als erstes muss mal wieder der Pixelgrapper ran:
		
		pg = new PixelGrabber(Bild, 0, 0, this.Bildbreite, this.Bildhoehe, Originalbild, 0, this.Bildbreite);
	    try
	    {
	            pg.grabPixels();
	    }
	    catch (InterruptedException e)
	    {
//	    	TODO debug
	       System.err.println("interrupted waiting for pixels!");

	    }
	    
	    /*
	     * Die Linien werden mit Hilfe des Satz des Pythagoras und
	     * dem Strahlensatz hezeichnet. Bei der Implementierung
	     * stellte sich heraus, das einige Flle unterschieden werden
	     * mussten. Siehe if - Statements
	     */
		
	    while(!Linien.empty())
		{
	    	// Linien Zeichnen Tool
	    	 //POWERTEST
	    	setPowertest();
	    	LinieInformationen li = (LinieInformationen)Linien.pop();
	    	 //POWERTEST
	    	setPowertest();
			int Richtungsfaktor = 1;
			 //POWERTEST
	    	setPowertest();
			int x1 = li.getx1();
			 //POWERTEST
	    	setPowertest();
			int x2 = li.getx2();
			 //POWERTEST
	    	setPowertest();
			int y1 = li.gety1();
			 //POWERTEST
	    	setPowertest();
			int y2 = li.gety2();
		
			 //POWERTEST
	    	setPowertest();
	    	/*
	    	 * Linienlnge mit Satz des Pythagoras errechnen
	    	 * Lnge = Wurzel((x2-x1)+(y2-y1));
	    	 */
			double Linienlaenge = Math.sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
			 //POWERTEST
	    	setPowertest();
	    	 //POWERTEST
	    	setPowertest(); 
	    	
	    	/*
	    	 * Fall 1:
	    	 * Drehung der Koordinaten wenn Position x1 hinter Position x2
	    	 * liegt ( bei Linie die so aussieht: / ; oder wenn die Linie 
	    	 * wagerecht ist y1 aber unterhalb von y2 ist.
	    	 */
			if (((x1 > x2) && (y1 > y2)) || ((x1 == x2) && (y1 > y2))) 
			{
				//powertest
				operationenZahl=operationenZahl+6;
			  int Eimer = x1;
			  x1 = x2;
			  x2 = Eimer;
			  Eimer = y1;
			  y1 = y2;
			  y2 = Eimer;
			}
			/*
			 * Fall 2:
			 * Drehung der Koordinaten wenn sowohl x1>x2 als auch y1>y2
			 * ist, oder wenn Linie senkrecht, aber die x1 Position
			 * hinter der x2 Position liegt.  
			 */
			else if (((x1 > x2) && (y1 < y2)) || ((y1 == y2) && (x1 > x2)))
			{
//				powertest
				operationenZahl=operationenZahl+6;
			  int Eimer = x1;
			  x1 = x2;
			  x2 = Eimer;
			  Eimer = y1;
			  y1 = y2;
			  y2 = Eimer;
			}
//			POWERTEST
	    	setPowertest();
	    	/*
	    	 * Fall 3:
	    	 * Position y1 ist unter y2 ( geometrisch gesehen ), die
	    	 * x Positionen stimmen aber. ( Aussehen der Linie \ )
	    	 * Der Faktor, in welche Richtung die Linie gezeichnet
	    	 * werden soll, wird negativ gesetzt. 
	    	 */
			if ((x1 < x2) && (y1 > y2))
			{
//				POWERTEST
		    	setPowertest();
			  Richtungsfaktor = -1;
			}

			
			
			for(int i= 0 ; i< (new Double(Linienlaenge)).intValue();i++)
			{
			       
				/*
				 * Strahlensatz anwenden.
				 */
				//SC = ((SQR(((ABS(x2 - x1)) ^ 2) + ((ABS(y2 - y1)) ^ 2))) / Linienlaenge) * I;
				double SC = (Math.sqrt((Math.abs(x2 - x1) * Math.abs(x2 - x1)) + ((Math.abs(y2 - y1)*Math.abs(y2 - y1)))) / Linienlaenge) * i;
				double SD = Math.sqrt((Math.abs(x2 - x1)*Math.abs(x2 - x1)) + (Math.abs(y2 - y1)*Math.abs(y2 - y1)));
				
				double BD = Math.abs(y2 - y1);
				
				double AC = BD * SC;
				AC = AC / SD;
				
				double SA = Math.sqrt((SC*SC) - (AC*AC));
				
				
				int neux = (new Double(x1+SA)).intValue();
				int neuy = (new Double(y1+(AC * Richtungsfaktor))).intValue();
				//powertest
				operationenZahl +=8;
			
//				POWERTEST
		    	setPowertest();
				if(neux>0 && neux<this.Bildbreite && neuy>0 && neuy<this.Bildhoehe)
				{//powetest
					operationenZahl+=2;
				  Originalbild[neuy*this.Bildbreite+neux] =  (Originalbild[neuy*this.Bildbreite+neux] & 0xFFFF00FF); 
				  Originalbild[neuy*this.Bildbreite+neux] = (Originalbild[neuy*this.Bildbreite+neux] | 0xFF00FF00);
				}
			
			}
		};		
	
//	  POWERTEST
    	setPowertest();
		newImg = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(Bildbreite, Bildhoehe, Originalbild, 0, Bildbreite) );
		
		powertest(Bildbreite, Bildhoehe);
	}
	
	/**
	 * Wird von der GUI aufgerufen. Es sollte die Schnittstelle
	 * IBildBearbeitung benutzt werden.
	 * @return Image; neu berechnetes Bild
	 */
	public Image getCalculatedImage() {return newImg;}

	/**
	 * In dieser Klasse liefert diese Methode nur das Originalbild
	 * zurck. Nhere Informationen siehe @see BerechnungHoughRaum#getMoreContrast(Image, int, int)
	 */
	public Image getMoreContrast(Image source,int width, int height) {return source;}
	
	/*
	 * Zur Auswertung von unserem Powertest und Ratiotest bentigt
	 */
	 long operationenZahl=0;
	    void setPowertest(){
	    	operationenZahl++;
	    }
	    private void powertest(int Bildbreite,int Bildhoehe){    	
	    	DateiFunktionen df = new DateiFunktionen("powertestHough.tmp");
	    	df.plotDateiLog();
	    	
			try{
				double x = Math.log(Double.valueOf((Bildbreite * Bildhoehe)));
				double y = Math.log(Double.valueOf(operationenZahl));
						df.dateiErgaenzen(x+ ", " +y );			
			}
			catch(Exception e){ e.printStackTrace();}	
		
	    }
	    public long getPowertestErg(){
	    	return operationenZahl;
	    }

}

/**
 * 
 * kappselt den Start- und End-Punkt mit seinen x und y Koordinaten
 *
 */
class LinieInformationen
{
   private int x1;
   private int y1;
   private int x2;
   private int y2;
   
   /**
    * Speichert die Linieninformationen
    * @param x1; Linke oberer Ecke der Linie
    * @param y1; Linke oberer Ecke der Linie
    * @param x2; Rechte untere Ecke der Linie
    * @param y2; Rechte untere Ecke der Linie
    */
   LinieInformationen(int x1,int y1,int x2,int y2)
   {
	   this.x1=x1;
	   this.x2=x2;
	   this.y1=y1;
	   this.y2=y2;
   }
   
   /**
    * Liefert die zu einer Linie gehrende X Koordinate
    *
    * @return int;x
    */
   public int getx1()
   {
	   return x1;
   }
   /**
    * Liefert die zu einer Linie gehrende Y Koordinate
    * @return int;y
    */
   public int gety1()
   {
	   return y1;
   }

   /**
    * Liefert die andere X Koordinate der Linie 
    * @return int
    */
   public int getx2()
   {
	   return x2;
   }

   /**
    * dito.
    * @return int
    */
   public int gety2()
   {
	   return y2;
   }


}

