/**
 * Klasse berechnet den Hough Raum.
 */
package Transformation.Hough;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.ImageObserver;
import java.awt.image.MemoryImageSource;
import java.awt.image.PixelGrabber;
import java.io.File;
import java.io.IOException;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import Transformation.importClass.DateiFunktionen;

/**
 * TODO hier sit die schwarz weis berechnung
 */

/**
 * berechniet das Image in schwarz weis, und gibt es als image zurpck
 * @author AZipser
 *
 */
public class BerechnungHoughRaum implements IBildBearbeitung {


	final double WINKELSCHRITTE = 1;
	final double FILTERGRENZWERT=10;
	final int RAUMBREITE;
	final int RAUMHOEHE;
	final int BILDBREITE;
	final int BILDHOEHE;
	final double SCHWELLWERT=0.95;
	

	Image OriginalBild;
	Image newImg;

    /**
     * Dem Konstruktor der Klasse wird das Originalbild als
     * Image Objekt bergeben.
     * @param src; Originalbild als Image Objekt
     * @param width; Bildbreite des Originalbild
     * @param height; Bildhhe des Originalbild
     */
	public BerechnungHoughRaum(Image src,int width,int height)
	{
		/*
		 * Die Raumbreite ( Breite des Akkumulators ) ist
		 * 2 x die Bilddiagonale ( fr negative und positive Abstnde ) 
		 * die Bildhhe ist 180 fr die Winkel -90 bis 90 Grad.
		 */
		RAUMBREITE=(new Long(Math.round(2*(Math.sqrt((width*width)+(height*height))))+1)).intValue();
		RAUMHOEHE=(new Long(Math.round(180*(1/WINKELSCHRITTE))+1).intValue());
		this.BILDBREITE=width;
		this.BILDHOEHE=height;
	    OriginalBild=src;

	}

	/**
	 * Berechnet <U>NUR</U> den Hough Raum und gibt diesen an unsere GUI
	 * zur&uuml;ck
	 * @return Image HoughRaum
	 */
	public Image getCalculatedImage()
	{
		return bildBerechnen();
	}

	/**
	 * Hauptmethode. Hier wird der Hough Raum berechnet.
	 * @return Image
	 */
	private Image bildBerechnen()
	{
		int[] HoughRaum = new int[180*2*(new Double(Math.ceil(Math.sqrt((this.BILDBREITE*this.BILDBREITE)+(this.BILDHOEHE*this.BILDHOEHE))))).intValue()];
		int[] ArrayBild = new int[this.BILDBREITE*this.BILDHOEHE];
		int w;
		int h;
		int winkel;

		

		/* Der Pixelgrabber holt uns einen Bildausschnit. In unserem
		* Falle aber das ganze Bild.
		*
		*/
		
		PixelGrabber pg = new PixelGrabber(OriginalBild, 0, 0, this.BILDBREITE, this.BILDHOEHE, ArrayBild, 0, this.BILDBREITE);
	    try
	    {
	            pg.grabPixels();
	    }
	    catch (InterruptedException e)
	    {
	       System.err.println("interrupted waiting for pixels!");
	       return null;
	    }

	    //POWERTEST
    	setPowertest();
    	/* 
    	 * Maximaler Abstand ausrechnen = AkkuBreite
    	 */
	    double MaximalerAbstand=Math.sqrt((this.BILDBREITE*this.BILDBREITE)+(this.BILDHOEHE*this.BILDHOEHE));;
	    
	    /*
	     * Als erstes machen den Hough Raum komplett schwarz.
	     * Dafr setzen wir jede Grundfarbe ( Rot, Grn, Blau ) auf
	     * 0 den AlphaWert ( fr die Helligkeit im Bild verantwortlich )
	     * setzen wir auf 255 ( hchster Wert ).
	     */
	    for(h=0;h<180;h++)
	    {
	    	
			for(w=0;w<(new Double((MaximalerAbstand*2))).intValue();w++)
			{
	//			POWERTEST
		    	setPowertest();
			  //Schwarzer Hintergrund
			  HoughRaum[2*h*(new Double(MaximalerAbstand)).intValue()+w]=0xFF000000;
			}
	    }
	    
//		POWERTEST
    	setPowertest();	    
	    //Hchster Wert
	    int maxPeak = 0;
	    //Hough Transformation anwenden
	    //1. Pixel fr Pixel im Originalbild abfragen
		for(h=0;h<BILDHOEHE;h++)
		{

			for(w=0;w<BILDBREITE;w++)
			{
//				POWERTEST
		    	setPowertest();
				//Das gefilterte Sobel Bild hat, wenn es eine Kante
				//an einer Stelle hervorgehoben hat, an dieser Stelle
				//bei allen drei Frabgebungen 255 stehen.
		    	//Da uns der Alphawert an dieser Stelle nicht
		    	//interessiert, wird dieser einfach ausgelassen:
		    	//Aufbau eines Pixels:
		    	//AlphaWert:RotAnteil:BlauAnteil:Grnanteil
				if((ArrayBild[h*this.BILDBREITE+w] & 0x00ffffff)==0x00ffffff)
				{
					//Berechne an dieser Stelle mit Winkel von -90 bis 90 mit
					//Hilfe von Hes' Normf. den Abstand zum Nullpunkt
					for(winkel=0;winkel<180;winkel++)
			    	{
//						POWERTEST
				    	setPowertest();
				    	//Hess'che Normalform
						long Abstand=(new Double(Math.ceil(w*Math.cos(winkel*(Math.PI/180))+h*Math.sin(winkel*(Math.PI/180))))).longValue();
						
						//Der HoughRaum hat einen schwarzen Hintergrund.  
						//Bisherige Anzahl der berschneidungen auslesen:

//						POWERTEST
				    	setPowertest();
				    	//Bisheriger Wert im Akkumulator entnehmen
						int val=(HoughRaum[(0+winkel)*2*(new Double(MaximalerAbstand)).intValue()+(new Double((Abstand+MaximalerAbstand))).intValue()]);
						//Filtere den Alphawert heraus
//						POWERTEST
				    	setPowertest();
						val=(val & 0x00FFFFFF);
					   //Eine berschneidung an diesem Punkt addieren...
//						POWERTEST
				    	setPowertest();
					   val=val+1;
					   
//						POWERTEST
				    	setPowertest();
					   if(maxPeak<val)
					   {
//							POWERTEST
					    	setPowertest();
						   maxPeak=val;
					   }
					   
					   //... die bisherigen Farbanteil reseten, um spter den
					   //addierten Wert mit OR Verknpfen zu knnen ...
//						POWERTEST
				    	setPowertest();
					   HoughRaum[(0+winkel)*2*(new Double(MaximalerAbstand)).intValue()+(new Double(Abstand+MaximalerAbstand)).intValue()] = (HoughRaum[(0+winkel)*2*(new Double(MaximalerAbstand)).intValue()+(new Double((Abstand+MaximalerAbstand))).intValue()] & 0xFF000000);
					   
//						POWERTEST
				    	setPowertest();
					   //..und zurckschreiben ( Um den Alphawert nicht zu berschreiben
					   //muss das zurckschreiben mittels einer OR (|) Verknpfung
					   //passieren.
					   HoughRaum[(0+winkel)*2*(new Double(MaximalerAbstand)).intValue()+(new Double(Abstand+MaximalerAbstand)).intValue()] = (HoughRaum[(0+winkel)*2*(new Double(MaximalerAbstand)).intValue()+(new Double((Abstand+MaximalerAbstand))).intValue()] | val);
					    
			    	}
						
					
				}			
			}
		}
		//			POWERTEST
    	setPowertest();
		powertest(this.BILDBREITE,  this.BILDBREITE);
		return Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(2*(new Double(MaximalerAbstand)).intValue(), 180, HoughRaum, 0, (new Double(MaximalerAbstand)).intValue()*2) );


	}

	/**
	 * Funktion hellt den Hough Raum etwas auf. Es ist mglich, das
	 * der Hough Raum ein bisschen dunkel erscheint. Diese Funktion
	 * sucht den maximalsten Ausschlag im Akkumulator und setzt
	 * diesen auf den Hchsten Farbwert ( in unserem Fall 255
	 * fr jeweils Rot, Grn, Blau ). Mittels des grten Ausschlags
	 * werden dann die anderen Ausschlge auf Farben verteilt.
	 * 
	 */
    public Image getMoreContrast(Image source,int width,int height)
    {

    	int[] ArrayBild = new int[width*height];
    	
		PixelGrabber pg = new PixelGrabber(source, 0, 0, width, height, ArrayBild, 0, width);
	    try
	    {
	            pg.grabPixels();

	            int min=255;
	            int max=0;
	            for(int y=0;y<height;y++)
	            {
	            	for(int x=0;x<width;x++)
	            	{
	            		int Farbe=ArrayBild[y*width+x];
	            		int rot=(Farbe & 0x00FF0000);
	            		int gruen=(Farbe & 0x0000FF00);
	            		int blau=(Farbe & 0x000000FF);
	            		int wert=(((new Double(Math.ceil((rot+gruen+blau)/3))).intValue()));
	            		if(wert>max)
	            		{
	            			max=wert;
	            		}
	            		if(wert<min && wert!=0)
	            		{
	            			min=wert;
	            		}
	            	}
	            }
	            //TODO debug
	            System.out.println("Minimalster Wert:" + min);
	            System.out.println("Maximalster Wert:"+ max);
	            for(int y=0;y<height;y++)
	            {
	            	for(int x=0;x<width;x++)
	            	{
	            		int Farbe=ArrayBild[y*width+x];
	            		ArrayBild[y*width+x]=0xFF000000;
	            		int rot=(Farbe & 0x00FF0000);
	            		int gruen=(Farbe & 0x0000FF00);
	            		int blau=(Farbe & 0x000000FF);
	            		double gw=Math.ceil((rot+gruen+blau)/3);
	            		gw=(255/max)*gw;
	            		int wert=(Farbe | ((new Double(gw)).intValue()));
	            		ArrayBild[y*width+x]=(ArrayBild[y*width+x] | wert);
	            		wert=(wert << 8);
	            		ArrayBild[y*width+x]=(ArrayBild[y*width+x] | wert);
	            		wert=(wert << 8);
	            		ArrayBild[y*width+x]=(ArrayBild[y*width+x] | wert);
	            		
	            	}
	            	//Farbe=((Farbe+1)%255);
	            }
	    }
	    catch (InterruptedException e)
	    {
	       System.err.println("interrupted waiting for pixels!");
	       return null;
	    }
    	
    	/*
    	 * Generiere aus dem Array ein neues Image Objekt.
    	 */
    	return Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(width, height, ArrayBild, 0, width ));
    }

    /**
     * Powertest
     */
    long operationenZahl=0;
    void setPowertest(){
    	operationenZahl++;
    }
    private void powertest(int Bildbreite,int Bildhoehe){    	

		DateiFunktionen df = new DateiFunktionen("powertestHR.tmp");
		df.plotDateiLog();
		try{
			double x = Math.log(Double.valueOf((Bildbreite * Bildhoehe)));
			double y = Math.log(Double.valueOf(operationenZahl));
			df.dateiErgaenzen(x+ ", " +y );			
			
		}
		catch(Exception e){ e.printStackTrace(); }	
		
    }
    public long getPowertestErg(){
    	return operationenZahl;
    }

}