package Transformation.importClass;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;


/**
 * @author Zipser
 * @param File datei
 * @exception IOException
 * @return String inhalt
 * @version 1.0
 * @since 14.10.2004
 * Liest eine Datei zeilenweise ein, und gibt den Inhalt als einen String wieder aus 
 * Inhalt hat folgende Struktur: 
 * 	Dateipfad
 * -----------
 *  Letzte nderung: tt.mm.jjjj  hh:mm:ss
 * -----------
 * inhalt der Datei
 */

public class DateiFunktionen {
	

	private String dateiPfad;
	private String dateiInhalt;
	private File datei;
	private int zeilen = 0;
	private GregorianCalendar cal;
	/**
	 * Default-Konstruktor
	 *
	 */
	public DateiFunktionen(){}
	
	/**
	 * Konstruktor
	 * @param name String (Dateiname)
	 */
	public DateiFunktionen(String name){
		this.datei = new File(name);
		dateiPfad = datei.getAbsolutePath();
	}

	
	/**
	 * Default konstruktor
	 * @param datei
	 */
	public DateiFunktionen(File datei)
	{
		this.datei = datei;
		dateiPfad = datei.getAbsolutePath();
		cal = new GregorianCalendar();
		cal.setTime(new Date(datei.lastModified()));
	}
	
	/**
	 * 
	 * @param File dieDatei 
	 * @param String text
	 * @throws IOException
	 */
	public void dateiSchreiben(String text) throws IOException
	{		
	    FileWriter fw = new FileWriter(datei);
		fw.write(text);
		fw.close();
	}
		
	/**
	 * 
	 * @param File dieDatei 
	 * @param String text
	 * @throws IOException
	 */
	public void dateiSchreiben(File dieDatei, String text) throws IOException
	{
	    FileWriter fw = new FileWriter(dieDatei);
		fw.write(text);
		fw.close();
	}

	/**
	 * vorhandene Datei mit String ergnzen
	 * @param name
	 */
	public void dateiErgaenzen(String inhalt){
	    FileWriter fw;
		try {
			fw = new FileWriter(datei,true);
			BufferedWriter bw = new BufferedWriter(fw);
			bw.write(inhalt);
			bw.newLine();
			bw.close();
			fw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * schreibt einen Header in die Datei, die fr den Plot bestimmt ist
	 *
	 */
	public void plotDateiLog(){
		if(!datei.exists()){
			dateiErgaenzen("TitleText: Test-Auswertung: Power-Test - manuell");			
			dateiErgaenzen("Grid: on" );
			dateiErgaenzen("Marks: various" );
			dateiErgaenzen("NumSets: 3" );
			dateiErgaenzen("Color: on" );
			dateiErgaenzen("XLog: on");
			dateiErgaenzen("YLog: on");
			dateiErgaenzen(" ");
			dateiErgaenzen("DataSet: Verlauf");
		}
	}
	
	
	public void Spaltenueberschrift(String inhalt)
	{
	  FileWriter fw;
	  try
	  {
		  if(!datei.exists())
		  {
			  fw=new FileWriter(datei);
			  BufferedWriter bw = new BufferedWriter(fw);
			  bw.write(inhalt);
			  bw.newLine();			  
		  }
	  }
	  catch(Exception e) 
	  {
		  e.printStackTrace();
	  }
	}
	
	
	
	
	/** 
	 * datei lesen und als stringarray zurckgeben
	 * @return String[][] x und y Werte getrennt
	 */
	public String[][] dateiLesen(String name){
		File datei = new File(name);
		String[][] arDateiInhalt =  new String[getZeilen(name+1)][2];
		
		String inhalt;
		
		if(!datei.exists()) { 
			System.err.println("Datei nicht gefunden");
			System.exit(1);
		}		
		try
		  {	   		   	
		   	BufferedReader bis = new BufferedReader(new FileReader(datei));
			int i=0;
		   	while(bis.ready()){
				inhalt = bis.readLine();
				arDateiInhalt[i][0] = inhalt.substring(0,(inhalt.indexOf("; ")-1));
				arDateiInhalt[i][1] = inhalt.substring((inhalt.indexOf("; ")+2));   	 		
				i++;
		   	}
			bis.close();
		  }	
		catch(IOException e){ System.err.println(e); }
		
		return arDateiInhalt;
	}
	

	/**
	 * liest den Dateiinhalt und schreibt in eine Interne Var.
	 * @return String
	 */
	private String dateiLesen()
	{
		int i=0;
		
	try
	  {	   		   	
	   	BufferedReader bis = new BufferedReader(new FileReader(datei));
		while(bis.ready()){
			if(i==0) {dateiInhalt = bis.readLine(); i++;}
	   	 	dateiInhalt += bis.readLine()+"\n";
	   	 	zeilen++;
		}
		bis.close();
	  }	
	catch(IOException e){ System.err.println(e); }
	return dateiInhalt;
	}
	

	
	/**
	 * Gibt das nderungsdatum der  Datei als String zurck
	 * @return String Datum (mm.tt.jjjj)
	 */
	public String getLastChangeDat()
	{
		return 	cal.get(Calendar.DATE) + "." + 
				(cal.get(Calendar.MONTH)+1) + "." +
				cal.get(Calendar.YEAR);		
	}
	
	/**
	 * gibt die letzte nderungszeit zurck 
	 * @return String time (hh:mm:ss)
	 */
	public String getLastChangeTime()
	{
		return 	cal.get(Calendar.HOUR_OF_DAY) + ":" +
				cal.get(Calendar.MINUTE) + ":" +
				cal.get(Calendar.SECOND);
	}
	

	/** <b>dateiInhalt</b>
	 * @return dateInhalt
	 */
	public String dateiInhalt()
	{
		return dateiLesen();
	}
	
		
	public boolean schreibRechte()
	{
		return datei.canWrite();
	}
	
	/**
	 * gibt die anzahl der zeilen zurck
	 * @return
	 */
	public int getZeilen()
	{
		return zeilen;
	}
	
	/**
	 * zhlt die anzahl zeilen in einer Datei
	 * @param name Dateiname
	 * @return
	 */
	public int getZeilen(String name){
		File datei = new File(name);
		int zeilenAnz=0;
		try
		  {	   		   	
		   	BufferedReader bis = new BufferedReader(new FileReader(datei));
			while(bis.ready()){
				bis.readLine();
				zeilenAnz++;
			}
			bis.close();
		  }	
		catch(IOException e){ System.err.println(e); }
		
		return zeilenAnz;
	}
	
}



